@@ -5,9 +5,9 @@
 
 /*
  * Mesa 3-D graphics library
- * Version:  6.5
+ * Version:  7.1
  *
- * Copyright (C) 1999-2006  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2007  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -42,10 +42,6 @@
 #include "mtypes.h"
 
 
-#ifdef __VMS
-#define _mesa_sprintf sprintf
-#endif
-
 /**********************************************************************/
 /** \name Internal functions */
 /*@{*/
@@ -104,7 +100,9 @@ _mesa_initialize_texture_object( struct gl_texture_object *obj,
           target == GL_TEXTURE_2D ||
           target == GL_TEXTURE_3D ||
           target == GL_TEXTURE_CUBE_MAP_ARB ||
-          target == GL_TEXTURE_RECTANGLE_NV);
+          target == GL_TEXTURE_RECTANGLE_NV ||
+          target == GL_TEXTURE_1D_ARRAY_EXT ||
+          target == GL_TEXTURE_2D_ARRAY_EXT);
 
    _mesa_bzero(obj, sizeof(*obj));
    /* init the non-zero fields */
@@ -138,13 +136,41 @@ _mesa_initialize_texture_object( struct gl_texture_object *obj,
    obj->CompareFunc = GL_LEQUAL;       /* ARB_shadow */
    obj->DepthMode = GL_LUMINANCE;      /* ARB_depth_texture */
    obj->ShadowAmbient = 0.0F;          /* ARB/SGIX_shadow_ambient */
-   _mesa_init_colortable(&obj->Palette);
+}
+
+
+/**
+ * Some texture initialization can't be finished until we know which
+ * target it's getting bound to (GL_TEXTURE_1D/2D/etc).
+ */
+static void
+finish_texture_init(GLcontext *ctx, GLenum target,
+                    struct gl_texture_object *obj)
+{
+   assert(obj->Target == 0);
+
+   if (target == GL_TEXTURE_RECTANGLE_NV) {
+      /* have to init wrap and filter state here - kind of klunky */
+      obj->WrapS = GL_CLAMP_TO_EDGE;
+      obj->WrapT = GL_CLAMP_TO_EDGE;
+      obj->WrapR = GL_CLAMP_TO_EDGE;
+      obj->MinFilter = GL_LINEAR;
+      if (ctx->Driver.TexParameter) {
+         static const GLfloat fparam_wrap[1] = {(GLfloat) GL_CLAMP_TO_EDGE};
+         static const GLfloat fparam_filter[1] = {(GLfloat) GL_LINEAR};
+         ctx->Driver.TexParameter(ctx, target, obj, GL_TEXTURE_WRAP_S, fparam_wrap);
+         ctx->Driver.TexParameter(ctx, target, obj, GL_TEXTURE_WRAP_T, fparam_wrap);
+         ctx->Driver.TexParameter(ctx, target, obj, GL_TEXTURE_WRAP_R, fparam_wrap);
+         ctx->Driver.TexParameter(ctx, target, obj, GL_TEXTURE_MIN_FILTER, fparam_filter);
+      }
+   }
 }
 
 
 /**
  * Deallocate a texture object struct.  It should have already been
  * removed from the texture object pool.
+ * Called via ctx->Driver.DeleteTexture() if not overriden by a driver.
  *
  * \param shared the shared GL state to which the object belongs.
  * \param texOjb the texture object to delete.
@@ -221,7 +247,7 @@ _mesa_copy_texture_object( struct gl_texture_object *dest,
    dest->_MaxLambda = src->_MaxLambda;
    dest->GenerateMipmap = src->GenerateMipmap;
    dest->Palette = src->Palette;
-   dest->Complete = src->Complete;
+   dest->_Complete = src->_Complete;
 }
 
 
@@ -239,6 +265,8 @@ valid_texture_object(const struct gl_texture_object *tex)
    case GL_TEXTURE_3D:
    case GL_TEXTURE_CUBE_MAP_ARB:
    case GL_TEXTURE_RECTANGLE_NV:
+   case GL_TEXTURE_1D_ARRAY_EXT:
+   case GL_TEXTURE_2D_ARRAY_EXT:
       return GL_TRUE;
    case 0x99:
       _mesa_problem(NULL, "invalid reference to a deleted texture object");
@@ -349,7 +377,7 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
    const GLint baseLevel = t->BaseLevel;
    GLint maxLog2 = 0, maxLevels = 0;
 
-   t->Complete = GL_TRUE;  /* be optimistic */
+   t->_Complete = GL_TRUE;  /* be optimistic */
 
    /* Always need the base level image */
    if (!t->Image[0][baseLevel]) {
@@ -357,7 +385,7 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
       _mesa_sprintf(s, "obj %p (%d) Image[baseLevel=%d] == NULL",
               (void *) t, t->Name, baseLevel);
       incomplete(t, s);
-      t->Complete = GL_FALSE;
+      t->_Complete = GL_FALSE;
       return;
    }
 
@@ -366,16 +394,18 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
        t->Image[0][baseLevel]->Height == 0 ||
        t->Image[0][baseLevel]->Depth == 0) {
       incomplete(t, "texture width = 0");
-      t->Complete = GL_FALSE;
+      t->_Complete = GL_FALSE;
       return;
    }
 
    /* Compute _MaxLevel */
-   if (t->Target == GL_TEXTURE_1D) {
+   if ((t->Target == GL_TEXTURE_1D) ||
+       (t->Target == GL_TEXTURE_1D_ARRAY_EXT)) {
       maxLog2 = t->Image[0][baseLevel]->WidthLog2;
       maxLevels = ctx->Const.MaxTextureLevels;
    }
-   else if (t->Target == GL_TEXTURE_2D) {
+   else if ((t->Target == GL_TEXTURE_2D) ||
+	    (t->Target == GL_TEXTURE_2D_ARRAY_EXT)) {
       maxLog2 = MAX2(t->Image[0][baseLevel]->WidthLog2,
                      t->Image[0][baseLevel]->HeightLog2);
       maxLevels = ctx->Const.MaxTextureLevels;
@@ -418,7 +448,7 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
 	 if (t->Image[face][baseLevel] == NULL ||
 	     t->Image[face][baseLevel]->Width2 != w ||
 	     t->Image[face][baseLevel]->Height2 != h) {
-	    t->Complete = GL_FALSE;
+	    t->_Complete = GL_FALSE;
 	    incomplete(t, "Non-quare cubemap image");
 	    return;
 	 }
@@ -435,7 +465,7 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
       GLint maxLevel = t->_MaxLevel;
 
       if (minLevel > maxLevel) {
-         t->Complete = GL_FALSE;
+         t->_Complete = GL_FALSE;
          incomplete(t, "minLevel > maxLevel");
          return;
       }
@@ -444,12 +474,12 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
       for (i = minLevel; i <= maxLevel; i++) {
          if (t->Image[0][i]) {
             if (t->Image[0][i]->TexFormat != t->Image[0][baseLevel]->TexFormat) {
-               t->Complete = GL_FALSE;
+               t->_Complete = GL_FALSE;
                incomplete(t, "Format[i] != Format[baseLevel]");
                return;
             }
             if (t->Image[0][i]->Border != t->Image[0][baseLevel]->Border) {
-               t->Complete = GL_FALSE;
+               t->_Complete = GL_FALSE;
                incomplete(t, "Border[i] != Border[baseLevel]");
                return;
             }
@@ -457,7 +487,8 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
       }
 
       /* Test things which depend on number of texture image dimensions */
-      if (t->Target == GL_TEXTURE_1D) {
+      if ((t->Target == GL_TEXTURE_1D) ||
+          (t->Target == GL_TEXTURE_1D_ARRAY_EXT)) {
          /* Test 1-D mipmaps */
          GLuint width = t->Image[0][baseLevel]->Width2;
          for (i = baseLevel + 1; i < maxLevels; i++) {
@@ -466,12 +497,12 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
             }
             if (i >= minLevel && i <= maxLevel) {
                if (!t->Image[0][i]) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "1D Image[0][i] == NULL");
                   return;
                }
                if (t->Image[0][i]->Width2 != width ) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "1D Image[0][i] bad width");
                   return;
                }
@@ -481,7 +512,8 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
             }
          }
       }
-      else if (t->Target == GL_TEXTURE_2D) {
+      else if ((t->Target == GL_TEXTURE_2D) ||
+               (t->Target == GL_TEXTURE_2D_ARRAY_EXT)) {
          /* Test 2-D mipmaps */
          GLuint width = t->Image[0][baseLevel]->Width2;
          GLuint height = t->Image[0][baseLevel]->Height2;
@@ -494,17 +526,17 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
             }
             if (i >= minLevel && i <= maxLevel) {
                if (!t->Image[0][i]) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "2D Image[0][i] == NULL");
                   return;
                }
                if (t->Image[0][i]->Width2 != width) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "2D Image[0][i] bad width");
                   return;
                }
                if (t->Image[0][i]->Height2 != height) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "2D Image[0][i] bad height");
                   return;
                }
@@ -532,26 +564,26 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
             if (i >= minLevel && i <= maxLevel) {
                if (!t->Image[0][i]) {
                   incomplete(t, "3D Image[0][i] == NULL");
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   return;
                }
                if (t->Image[0][i]->_BaseFormat == GL_DEPTH_COMPONENT) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "GL_DEPTH_COMPONENT only works with 1/2D tex");
                   return;
                }
                if (t->Image[0][i]->Width2 != width) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "3D Image[0][i] bad width");
                   return;
                }
                if (t->Image[0][i]->Height2 != height) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "3D Image[0][i] bad height");
                   return;
                }
                if (t->Image[0][i]->Depth2 != depth) {
-                  t->Complete = GL_FALSE;
+                  t->_Complete = GL_FALSE;
                   incomplete(t, "3D Image[0][i] bad depth");
                   return;
                }
@@ -577,20 +609,20 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
 	       for (face = 0; face < 6; face++) {
 		  /* check that we have images defined */
 		  if (!t->Image[face][i]) {
-		     t->Complete = GL_FALSE;
+		     t->_Complete = GL_FALSE;
 		     incomplete(t, "CubeMap Image[n][i] == NULL");
 		     return;
 		  }
 		  /* Don't support GL_DEPTH_COMPONENT for cube maps */
 		  if (t->Image[face][i]->_BaseFormat == GL_DEPTH_COMPONENT) {
-		     t->Complete = GL_FALSE;
+		     t->_Complete = GL_FALSE;
 		     incomplete(t, "GL_DEPTH_COMPONENT only works with 1/2D tex");
 		     return;
 		  }
 		  /* check that all six images have same size */
 		  if (t->Image[face][i]->Width2!=width || 
 		      t->Image[face][i]->Height2!=height) {
-		     t->Complete = GL_FALSE;
+		     t->_Complete = GL_FALSE;
 		     incomplete(t, "CubeMap Image[n][i] bad size");
 		     return;
 		  }
@@ -618,13 +650,6 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
 /** \name API functions */
 /*@{*/
 
-/**
- * Texture name generation lock.
- *
- * Used by _mesa_GenTextures() to guarantee that the generation and allocation
- * of texture IDs is atomic.
- */
-_glthread_DECLARE_STATIC_MUTEX(GenTexturesLock);
 
 /**
  * Generate texture names.
@@ -634,9 +659,9 @@ _glthread_DECLARE_STATIC_MUTEX(GenTexturesLock);
  *
  * \sa glGenTextures().
  *
- * While holding the GenTexturesLock lock, calls _mesa_HashFindFreeKeyBlock()
- * to find a block of free texture IDs which are stored in \p textures.
- * Corresponding empty texture objects are also generated.
+ * Calls _mesa_HashFindFreeKeyBlock() to find a block of free texture
+ * IDs which are stored in \p textures.  Corresponding empty texture
+ * objects are also generated.
  */ 
 void GLAPIENTRY
 _mesa_GenTextures( GLsizei n, GLuint *textures )
@@ -657,7 +682,7 @@ _mesa_GenTextures( GLsizei n, GLuint *textures )
    /*
     * This must be atomic (generation and allocation of texture IDs)
     */
-   _glthread_LOCK_MUTEX(GenTexturesLock);
+   _glthread_LOCK_MUTEX(ctx->Shared->Mutex);
 
    first = _mesa_HashFindFreeKeyBlock(ctx->Shared->TexObjects, n);
 
@@ -668,20 +693,18 @@ _mesa_GenTextures( GLsizei n, GLuint *textures )
       GLenum target = 0;
       texObj = (*ctx->Driver.NewTextureObject)( ctx, name, target);
       if (!texObj) {
-         _glthread_UNLOCK_MUTEX(GenTexturesLock);
+         _glthread_UNLOCK_MUTEX(ctx->Shared->Mutex);
          _mesa_error(ctx, GL_OUT_OF_MEMORY, "glGenTextures");
          return;
       }
 
       /* insert into hash table */
-      _glthread_LOCK_MUTEX(ctx->Shared->Mutex);
       _mesa_HashInsert(ctx->Shared->TexObjects, texObj->Name, texObj);
-      _glthread_UNLOCK_MUTEX(ctx->Shared->Mutex);
 
       textures[i] = name;
    }
 
-   _glthread_UNLOCK_MUTEX(GenTexturesLock);
+   _glthread_UNLOCK_MUTEX(ctx->Shared->Mutex);
 }
 
 
@@ -736,6 +759,12 @@ unbind_texobj_from_texunits(GLcontext *ctx, struct gl_texture_object *texObj)
       else if (texObj == unit->CurrentRect) {
          _mesa_reference_texobj(&unit->CurrentRect, ctx->Shared->DefaultRect);
       }
+      else if (texObj == unit->Current1DArray) {
+         _mesa_reference_texobj(&unit->Current1DArray, ctx->Shared->Default1DArray);
+      }
+      else if (texObj == unit->Current2DArray) {
+         _mesa_reference_texobj(&unit->Current2DArray, ctx->Shared->Default2DArray);
+      }
    }
 }
 
@@ -825,38 +854,45 @@ _mesa_BindTexture( GLenum target, GLuint texName )
    GET_CURRENT_CONTEXT(ctx);
    const GLuint unit = ctx->Texture.CurrentUnit;
    struct gl_texture_unit *texUnit = &ctx->Texture.Unit[unit];
-   struct gl_texture_object *newTexObj = NULL;
+   struct gl_texture_object *newTexObj = NULL, *defaultTexObj = NULL;
    ASSERT_OUTSIDE_BEGIN_END(ctx);
 
    if (MESA_VERBOSE & (VERBOSE_API|VERBOSE_TEXTURE))
       _mesa_debug(ctx, "glBindTexture %s %d\n",
                   _mesa_lookup_enum_by_nr(target), (GLint) texName);
 
+   switch (target) {
+   case GL_TEXTURE_1D:
+      defaultTexObj = ctx->Shared->Default1D;
+      break;
+   case GL_TEXTURE_2D:
+      defaultTexObj = ctx->Shared->Default2D;
+      break;
+   case GL_TEXTURE_3D:
+      defaultTexObj = ctx->Shared->Default3D;
+      break;
+   case GL_TEXTURE_CUBE_MAP_ARB:
+      defaultTexObj = ctx->Shared->DefaultCubeMap;
+      break;
+   case GL_TEXTURE_RECTANGLE_NV:
+      defaultTexObj = ctx->Shared->DefaultRect;
+      break;
+   case GL_TEXTURE_1D_ARRAY_EXT:
+      defaultTexObj = ctx->Shared->Default1DArray;
+      break;
+   case GL_TEXTURE_2D_ARRAY_EXT:
+      defaultTexObj = ctx->Shared->Default2DArray;
+      break;
+   default:
+      _mesa_error(ctx, GL_INVALID_ENUM, "glBindTexture(target)");
+      return;
+   }
+
    /*
     * Get pointer to new texture object (newTexObj)
     */
    if (texName == 0) {
-      /* newTexObj = a default texture object */
-      switch (target) {
-         case GL_TEXTURE_1D:
-            newTexObj = ctx->Shared->Default1D;
-            break;
-         case GL_TEXTURE_2D:
-            newTexObj = ctx->Shared->Default2D;
-            break;
-         case GL_TEXTURE_3D:
-            newTexObj = ctx->Shared->Default3D;
-            break;
-         case GL_TEXTURE_CUBE_MAP_ARB:
-            newTexObj = ctx->Shared->DefaultCubeMap;
-            break;
-         case GL_TEXTURE_RECTANGLE_NV:
-            newTexObj = ctx->Shared->DefaultRect;
-            break;
-         default:
-            _mesa_error(ctx, GL_INVALID_ENUM, "glBindTexture(target)");
-            return;
-      }
+      newTexObj = defaultTexObj;
    }
    else {
       /* non-default texture object */
@@ -864,25 +900,13 @@ _mesa_BindTexture( GLenum target, GLuint texName )
       if (newTexObj) {
          /* error checking */
          if (newTexObj->Target != 0 && newTexObj->Target != target) {
-            /* the named texture object's dimensions don't match the target */
+            /* the named texture object's target doesn't match the given target */
             _mesa_error( ctx, GL_INVALID_OPERATION,
-                         "glBindTexture(wrong dimensionality)" );
+                         "glBindTexture(target mismatch)" );
             return;
          }
-         if (newTexObj->Target == 0 && target == GL_TEXTURE_RECTANGLE_NV) {
-            /* have to init wrap and filter state here - kind of klunky */
-            newTexObj->WrapS = GL_CLAMP_TO_EDGE;
-            newTexObj->WrapT = GL_CLAMP_TO_EDGE;
-            newTexObj->WrapR = GL_CLAMP_TO_EDGE;
-            newTexObj->MinFilter = GL_LINEAR;
-            if (ctx->Driver.TexParameter) {
-               static const GLfloat fparam_wrap[1] = {(GLfloat) GL_CLAMP_TO_EDGE};
-               static const GLfloat fparam_filter[1] = {(GLfloat) GL_LINEAR};
-               (*ctx->Driver.TexParameter)( ctx, target, newTexObj, GL_TEXTURE_WRAP_S, fparam_wrap );
-               (*ctx->Driver.TexParameter)( ctx, target, newTexObj, GL_TEXTURE_WRAP_T, fparam_wrap );
-               (*ctx->Driver.TexParameter)( ctx, target, newTexObj, GL_TEXTURE_WRAP_R, fparam_wrap );
-               (*ctx->Driver.TexParameter)( ctx, target, newTexObj, GL_TEXTURE_MIN_FILTER, fparam_filter );
-            }
+         if (newTexObj->Target == 0) {
+            finish_texture_init(ctx, target, newTexObj);
          }
       }
       else {
@@ -926,6 +950,12 @@ _mesa_BindTexture( GLenum target, GLuint texName )
       case GL_TEXTURE_RECTANGLE_NV:
          _mesa_reference_texobj(&texUnit->CurrentRect, newTexObj);
          break;
+      case GL_TEXTURE_1D_ARRAY_EXT:
+         texUnit->Current1DArray = newTexObj;
+         break;
+      case GL_TEXTURE_2D_ARRAY_EXT:
+         texUnit->Current2DArray = newTexObj;
+         break;
       default:
          /* Bad target should be caught above */
          _mesa_problem(ctx, "bad target in BindTexture");
@@ -1070,14 +1100,21 @@ _mesa_IsTexture( GLuint texture )
    return t && t->Target;
 }
 
-/* Simplest implementation of texture locking: Grab the a new mutex in
+
+/**
+ * Simplest implementation of texture locking: Grab the a new mutex in
  * the shared context.  Examine the shared context state timestamp and
  * if there has been a change, set the appropriate bits in
  * ctx->NewState.
  *
- * See also _mesa_lock/unlock_texture in texobj.h
+ * This is used to deal with synchronizing things when a texture object
+ * is used/modified by different contexts (or threads) which are sharing
+ * the texture.
+ *
+ * See also _mesa_lock/unlock_texture() in teximage.h
  */
-void _mesa_lock_context_textures( GLcontext *ctx )
+void
+_mesa_lock_context_textures( GLcontext *ctx )
 {
    _glthread_LOCK_MUTEX(ctx->Shared->TexMutex);
 
@@ -1088,7 +1125,8 @@ void _mesa_lock_context_textures( GLcontext *ctx )
 }
 
 
-void _mesa_unlock_context_textures( GLcontext *ctx )
+void
+_mesa_unlock_context_textures( GLcontext *ctx )
 {
    assert(ctx->Shared->TextureStateStamp == ctx->TextureStateTimestamp);
    _glthread_UNLOCK_MUTEX(ctx->Shared->TexMutex);
